#!/usr/bin/python

# **************************************************************************
# ***     European Southern Observatory - Science Archive Facility       ***
# ***                             binary_spec1.py                        ***
# ***              Example of how to create a 1d FITS spectrum           ***
# ***                   compliant to the ESO SDP Standard                ***
# ***                              2017-Oct-23                           ***
# ***   For questions contact usd@eso.org (Subject: 'Phase3 spectra')    ***
# **************************************************************************

from astropy.io import fits

orig = fits.open('spec1.fits') #Note: This FITS (single extension) contains a table consiting of 1 array: flux.

val = orig[0].data
el = len(val) # Number of data points
prihdr = orig[0].header

## Flux array
flux = val

## creating the error array - Filled with 0 here, please change!
err = [0.0]*el #or err = orig[1].data in case errors are stored in the FITS extension
form = str(el)+'E'

## Creating the wavelength array from the WCS
w0 = prihdr['CRVAL1']
cdel = prihdr['CDELT1']
crpix1 = prihdr['CRPIX1']

w1 = (1-crpix1)*cdel + w0
wav = []
for i in range(el):
  wav.append(w1)
  w1 = w1 + cdel

## Creating the data extension table. Units need to be modified!
table_hdu = fits.BinTableHDU.from_columns(
  [fits.Column(name='WAVE', format=form, unit='angstrom', array=[wav]), 
  fits.Column(name='FLUX', format=form, unit='10**(-17)erg.cm**(-2).s**(-1).angstrom**(-1)', array=[flux]),
  fits.Column(name='ERR', format=form, unit='10**(-17)erg.cm**(-2).s**(-1).angstrom**(-1)', array=[err])],
  nrows = 1)

## add/remove/modify additional required Phase 3 keywords from the Primary header.
prihdr['PRODCATG'] = ('SCIENCE.SPECTRUM', 'Data product category')
prihdr['ORIGIN'] = ('ESO', 'European Southern Observatory')
prihdr['FLUXCAL'] = ('ABSOLUTE', 'Type of flux calibration')

## Other required keywords in the primary header, the correct values must be inserted before uncommenting: 
## OBJECT, TELESCOP, INSTRUME, RA, DEC, EXPTIME, TEXPTIME, MJD-OBS, MJD-END,
## EXT_OBJ, SPEC_RES, M_EPOCH, DISPELEM, OBSTECH, SPECSYS, EQUINOX, 
## WAVELMIN, WAVELMAX, SPEC_BIN, SNR, TOT_FLUX, FLUXERR, CONTNORM,
## OBID1, PROG_ID, PROVi (where i=1,...,n), NCOMBINE, REFERENC
# 
# prihdr['OBJECT'] = (' ', 'Target designation')
# prihdr['TELESCOP'] = ('ESO-VLT-U#', 'ESO Telescope designation')
# prihdr['INSTRUME'] = (' ', 'Instrument name')
# prihdr['RA'] = (  , '[deg] Spectroscopic target position (J2000.0)')
# prihdr['DEC'] = ( , '[deg] Spectroscopic target position (J2000.0)')
# prihdr['EXPTIME'] = ( , 'Total integration time per pixel (s)')
# prihdr['TEXPTIME'] = ( , 'Total integration time of all exposures (s)')
# prihdr['MJD-OBS'] = ( , 'Start of observations (days)')
# prihdr['MJD-END'] = ( , 'End of observations (days)')
# prihdr['EXT_OBJ'] = ( , 'TRUE if extended')
# prihdr['SPEC_RES'] = ( , 'Resolving power')
# prihdr['M_EPOCH'] = ( , 'TRUE if resulting from multiple epochs')
# prihdr['DISPELEM'] = (' ', 'Dispersive element name')
# prihdr['OBSTECH'] = (' ', 'Tequnique of observation')
# prihdr['SPECSYS'] = (' ', 'Reference frame for spectral coordinates')

prihdr['EQUINOX'] = ( 2000., 'Standard FK5 (years)')

# Note: min/max values of the wavelength arrays could be easily computed in python: min(wav), max(wav).
# This is not done here because the user must provide not just the min and max values, but the values
# within which the scientifically valid part of the spectrum lies
# (e.g. spurious flux values introduced to fill-in the array to a given length should not be considered, etc.).
prihdr['WAVELMIN'] = ( min(wav)*1000, '[nm] Minimum wavelength') # min(wav) converted into nm
prihdr['WAVELMAX'] = ( max(wav)*1000, '[nm] Maximum wavelength')  # max(wav) converted into nm

prihdr['SPEC_BIN'] = ( (prihdr['WAVELMAX'] - prihdr['WAVELMIN'])/(el -1), '[nm] Wavelength bin size') #Note: the average bin size
# or prihdr['SPEC_BIN'] = ( , '[nm] Wavelength bin size') # cdel converted into nm

# prihdr['SNR'] = ( , 'Signal to noise ratio per pixel') 
# prihdr['TOT_FLUX'] = ( , 'True if all src flux is captured')
# prihdr['FLUXERR'] = ( , 'Uncertainty in flux scale')
# prihdr['CONTNORM'] = ( , 'TRUE if normalised to the continuum')
#
# prihdr['OBID1'] = ( , 'Observation block ID')
# prihdr['PROG_ID'] = (' ', 'ESO programme identification')
# prihdr['NCOMBINE'] = ( , '# of combined raw science frames')
# prihdr['REFERENC'] = (' ', 'Bibliographic reference')
# prihdr['PROV1'] = (' ' , 'Originating science file')
#
## Remove keywords using the following command:
del prihdr['BUNIT']
del prihdr['CRPIX1']
del prihdr['CRVAL1']
del prihdr ['CDELT1']
del prihdr ['CD1_1']
del prihdr['CTYPE1']
del prihdr['CUNIT1']

## To rename keywords:
# prihdr.rename_keyword('DEC_TARG', 'DEC')
#
## To modify only the comment: 
# prihdr.comments['DEC'] = '[deg] Spectroscopic target position (J2000.0)'  
#
## Encoding associated files
# prihdr['ASSON1'] = (' ', 'Name of associated file')
# prihdr['ASSOC1'] = (' ', 'Category of associated file') # Aplicable only for non-FITS files
# prihdr['ASSOM1'] = (' ', 'md5sum of ASSON1') # Aplicable only for non-FITS files


## Creating the structure of the file: empty primary HDU followed by the extension with the data.
phdu = fits.PrimaryHDU(header=prihdr) 
hdulist = fits.HDUList([phdu, table_hdu])

## Modify the extension header by adding the required Phase 3 keywords:
exthdr = hdulist[1].header

exthdr['VOPUB'] = ('ESO/SAF', 'VO Publishing Authority')
exthdr['VOCLASS'] = ('SPECTRUM V2.0', 'VO Data Model')
exthdr['NELEM'] = (el, 'Length of the data array')

#Note: uncomment one of the two following lines, depending if the wavelength provided is measured in vacuum or in air
#exthdr['TUCD1'] = ('em.wl;obs.atmos', 'Air wavelength')
#exthdr['TUCD1'] = ('em.wl', 'Vacuum wavelength')
exthdr['TUCD2'] = ('phot.flux.density;em.wl;meta.main', 'UCD filed 2')
exthdr['TUCD3'] = ('stat.error;phot.flux.density;meta.main', 'UCD filed 3')

exthdr['TUTYP1'] = ('spec:Data.SpectralAxis.Value', 'IVOA data model element for field 1')
exthdr['TUTYP2'] = ('spec:Data.FluxAxis.Value', 'IVOA data model element for field 2')
exthdr['TUTYP3'] = ('spec:Data.FluxAxis.Accuracy.StatError', 'IVOA data model element for field 3')

exthdr['EXTNAME'] = ('SPECTRUM', 'FITS extension name') 

## propagating RA, DEC and OBJECT from the primary header. Such keywords need to be in the PHDU first.
#exthdr['RA'] = (prihdr['RA'], '[deg] Spectroscopic target position (J2000.0)')
#exthdr['DEC'] = (prihdr['DEC'], '[deg] Spectroscopic target position (J2000.0)')
#exthdr['OBJECT'] = (prihdr['OBJECT'], 'Target designation')

## Setting TDMIN/MAX1 to the value of the first element in the first column. 
#Note: TDMIN1/TDMAX1 can be different from the WAVELMIN/MAX values.
exthdr['TDMIN1'] = (min(wav), 'Start in spectral coord.')
exthdr['TDMAX1'] = (max(wav), 'Stop in spectral coord.')

## Calculating the value for TELAPSE. The values of MJD-OBS/END in the primary header are needed.
#t_el = (prihdr['MJD-END']-prihdr['MJD-OBS'])*86400
#exthdr['TELAPSE'] = (tel, '[s] total elapsed time')

## Calculating TMID. The values of MJD-OBS/END in the primary header are needed.
#tmid = (prihdr['MJD-END']+prihdr['MJD-OBS'])/2.0
#exthdr['TMID'] = (tmid, 'MJD mid exposure')

## Calculating SPEC_VAL and SPEC_BW. The values of WAVELMIN/MAX in the primary header are needed. 
#wmin = prihdr['WAVELMIN']
#wmax = prihdr['WAVELMAX']
#wmean = (wmin+wmax)/2
#wrange = wmax-wmin

#exthdr['SPEC_VAL'] = (wmean, '[nm] Mean Wavelength')    
#exthdr['SPEC_BW'] = (wrange, '[nm] Bandpass Width Wmax - Wmin')

## Other required keywords in the extension header: 
## TITLE, APERTURE, modify the following two lines with the correct value
# exthdr['TITLE'] = (' ', 'Dataset title')
# exthdr['APERTURE'] = ( , '[deg] Aperture diameter')


hdulist[1].header = exthdr

## Saving the modified spectrum into a new file

hdulist.writeto('conv_spec1.fits', checksum=True)

